from smartwallet.ui.app import SmartWalletApp, _get_version


def test_app_instantiation() -> None:
    app = SmartWalletApp()
    assert app.title == f"SmartWallet v{_get_version()}"


def test_run_function_exists() -> None:
    from smartwallet.main import run

    assert callable(run)
